import { inject } from '@angular/core';
import { Logger } from '@smallstack/core-common';
import { TranslationStore } from '@smallstack/i18n-components';
import { FrontendTypeRegistry, RxCollectionService } from '@smallstack/typesystem-client';
import { isNonEmptyString } from '@smallstack/utils';
import Fuse from 'fuse.js';
import { combineLatest, filter, switchMap, of, map, shareReplay } from 'rxjs';

/** returns available options based on a type schema and a search text */
function getSelectOptions$(schema$, searchText$) {
    const frontendTypeRegistry = inject(FrontendTypeRegistry);
    const rxCollectionService = inject(RxCollectionService);
    const translationStore = inject(TranslationStore);
    return combineLatest([schema$, searchText$]).pipe(filter(([schema, searchText]) => schema !== undefined), switchMap(([schema, searchText]) => {
        // values given? easy
        if (schema["x-schema-form"]?.["values"])
            return of([schema["x-schema-form"]?.["values"], searchText, false]);
        // support enum as fallback
        else if (schema.enum)
            return of([schema.enum.map((val) => ({ value: val, label: "" + val })), searchText, false]);
        // if type is given, load all entries
        else if (schema["x-schema-form"]?.type) {
            return rxCollectionService
                .getCollection$(schema["x-schema-form"].type, { tenantId: schema["x-schema-form"].typeTenantId })
                .pipe(switchMap(async (collection) => {
                await collection.replicationState.awaitInSync();
                return collection;
            }), switchMap((collection) => {
                return collection.rxCollection.find().$;
            }), switchMap(async (values) => {
                const type = await frontendTypeRegistry.getTypeByPath(schema["x-schema-form"].type);
                return values.map((val) => ({
                    value: val.id,
                    label: frontendTypeRegistry.getRepresentation(type, val.toJSON())
                }));
            }), map((values) => [values, searchText, true]));
        }
        // else return empty array
        Logger.warning("SelectRxJSHelper", "Neither values nor enum nor type given for select widget. Schema:", schema);
        return of([[], searchText, false]);
    }), map(([values, searchText, shallBeSorted]) => {
        values.map((value) => {
            value.label = translationStore.translate(value.label);
            return value;
        });
        if (shallBeSorted === true)
            return [
                values.sort((a, b) => {
                    if (a.label === b.label)
                        return 0;
                    if (a.label === undefined)
                        return -1;
                    if (b.label === undefined)
                        return 1;
                    return a.label.localeCompare(b.label);
                }),
                searchText
            ];
        else
            return [values, searchText];
    }), map(([values, searchText]) => {
        if (isNonEmptyString(searchText)) {
            const fuse = new Fuse(values, {
                keys: [
                    { name: "label", weight: 7 },
                    { name: "value", weight: 3 }
                ],
                isCaseSensitive: false,
                shouldSort: true,
                threshold: 0.2
            });
            return fuse.search(searchText).map((result) => result.item);
        }
        return values;
    }), shareReplay(1));
}

export { getSelectOptions$ as g };
//# sourceMappingURL=smallstack-form-widgets-select-rxjs-helpers-3bc7d032.mjs.map
